/**
 * Настройки
 * @type {{webHook: string, apiUrl: string, token: string}}
 */
const config = {
  token: "your_token_bot",
  webHook: "https://script.google.com/macros/s/---------/exec",
  apiUrl: "https://api.telegram.org/bot"
};

/**
 * Переменная для хранения данных
 */
let data;

/**
 * Получаем данные от Телеграм
 * @param request
 */
function doPost(request) {
  // проверяем что запрос от телеграм с токеном
  if (request.parameter.token === config.token) {
    // записываем данные
    data = JSON.parse(request.postData.contents);
    // вызываем функцию
    webHook();
  }
}

/**
 * Функция обработки данных
 */
function webHook() {
  // объявим переменные
  let text, entities = null;
  // проверим что это сообщение
  if ('message' in data) {
    // если это текстовое сообщение
    if ('text' in data.message) {
      // запишем текст
      text = data.message.text;
      // если есть форматирование
      if ('entities' in data.message) {
        // запишем форматирование
        entities = data.message.entities;
      }
    } else {
      // если есть описание объекта
      if ('caption' in data.message) {
        // запишем текст
        text = data.message.caption;
        // если есть форматирование
        if ('caption_entities' in data.message) {
          // запишем форматирование
          entities = data.message.caption_entities;
        }
      }
    }
  }
  // если есть текст
  if(text !== null) {
    // передаем на обработку
    checkText(text, entities);
  }
}

/**
 * Проверяем необходимость проверки
 * @param text
 * @param entities
 */
function checkText(text, entities) {
  // массив заперещенного форматирования
  const off = ['text_link', 'url', 'email'];
  // если есть форматирование
  if (entities !== null) {
    // получаем массив запрещенного
    let prohibited = entities.filter(item => off.includes(item.type));
    // получаем массив допустимого
    let admissible = entities.filter(item => !off.includes(item.type));
    // если есть что заменить
    if (prohibited.length) {
      // отправляем на обработку
      prepareText(text, prohibited, admissible);
    }
  }
}

/**
 * Заменяем запрещенное
 * @param text
 * @param prohibited
 * @param admissible
 */
function prepareText(text, prohibited, admissible) {
  // перебираем запрещенное и заменяем текст звездочкой
  prohibited.forEach(function (item) {
    // обрабатываем
    text = replaceText(text, item);
  });
  // отправим отформатированный текст
  sendFormattedText(text, admissible);
}

/**
 * Заменяем
 * @param text
 * @param formate
 * @returns {string}
 */
function replaceText(text, formate) {
  // получим текст до
  let formatting = text.substring(formate.offset, formate.offset + formate.length).replace(/[^\s]/g, "*");
  return text.substring(0, formate.offset) + formatting + text.substring(formate.offset + formate.length);
}

/**
 *
 * @param text_
 * @param admissible
 */
function sendFormattedText(text_, admissible) {
  deleteMessage(data.message.chat.id, data.message.message_id);
  let text = prepareMessageWithEntities(text_, admissible);
  let user = (data.message.from.first_name + " " + data.message.from.last_name).trim();
  sendMessage(data.message.chat.id, "Сообщение от пользователя: <b>" + user + "</b> было отредактрировано:\n-----\n" + text);
}

/**
 * Форматирование текста
 * @param text
 * @param entities
 * @returns {*}
 */
function prepareMessageWithEntities(text, entities) {
  // проверяем наличие форматирования
  if (entities !== null && entities.length > 0) {
    // готовим переменную в нее будем добавлять
    let prepareText = "";
    // перебираем форматирование
    entities.forEach(function (entity, idx, arr) {
      // добавляем все что между форматированием
      if (entity.offset > 0) {
        /*
          * старт = если начало больше 0 и это первый элемент то берем сначала с нуля
          * если не первый то берем сразу после предыдущего элемента
          *
          * длина = это разница между стартом и текущим началом
          */
        // определяем начало
        let start = (idx === 0)
          ? 0
          : (arr[idx - 1].offset + arr[idx - 1].length);
        // определяем длину
        let length = entity.offset - start;
        // добавляем
        prepareText = prepareText + text.substr(start, length);
      }
      // выбираем текущий элемент форматирования
      let charts = text.substr(entity.offset, entity.length);
      // обрамляем в необходимый формат
      if (entity.type === "bold") {
        // полужирный
        charts = "<b>" + charts + "</b>";
      } else if (entity.type === "italic") {
        // курсив
        charts = "<i>" + charts + "</i>";
      } else if (entity.type === "code") {
        // код
        charts = "<code>" + charts + "</code>";
      } else if (entity.type === "pre") {
        // inline код
        charts = "<pre>" + charts + "</pre>";
      } else if (entity.type === "strikethrough") {
        // зачеркнутый
        charts = "<s>" + charts + "</s>";
      } else if (entity.type === "underline") {
        // подчеркнутый
        charts = "<u>" + charts + "</u>";
      } else if (entity.type === "spoiler") {
        // скрытый
        charts = "<tg-spoiler>" + charts + "</tg-spoiler>";
      } else if (entity.type === "text_link") {
        // ссылка текстовая
        charts = "<a href='" + entity.url + "'>" + charts + "</a>";
      }
      // добавляем в переменную
      prepareText = prepareText + charts;
    });
    // добавляем остатки текста если такие есть
    prepareText = prepareText + text.substr((entities[entities.length - 1].offset + entities[entities.length - 1].length));
    // возвращаем результат
    return prepareText;
  }
  // по умолчанию вернем не форматированный текст
  return text;
}

/**
 * Получаем информацию о Вебхуке
 */
function getWebHookInfo() {
  let response = UrlFetchApp.fetch(config.apiUrl + config.token + "/getWebHookInfo");
  console.log(response.getContentText());
}

/**
 * Устанавливаем Вебхук
 */
function setWebHook() {
  let response = UrlFetchApp.fetch(config.apiUrl + config.token + "/setWebHook?url=" + config.webHook + "?token=" + config.token);
  console.log(response.getContentText());
}

/**
 * Удалим Вебхук
 */
function deleteWebHook() {
  let response = UrlFetchApp.fetch(config.apiUrl + config.token + "/deleteWebhook?drop_pending_updates=true");
  console.log(response.getContentText());
}

/**
 * Удалим сообщение
 * @param chat_id
 * @param message_id
 */
function deleteMessage(chat_id, message_id) {
  let payload = {
    method: "deleteMessage",
    chat_id: String(chat_id),
    message_id: String(message_id),
  };
  query(payload);
}

/**
 * Отправляем сообщение
 * @param chat_id
 * @param text
 * @returns {*}
 */
function sendMessage(chat_id, text) {
  // готовим данные
  let payload = {
    method: "sendMessage",
    chat_id: String(chat_id),
    text: text,
    parse_mode: "HTML",
  };
  // вернем результат отправки
  return query(payload);
}

/**
 * Запрос в Телеграм
 * @param payload
 * @returns {any}
 */
function query(payload) {
  // готовим данные
  let data = {
    method: "post",
    payload: payload
  };
  // вернем объект ответа
  return JSON.parse(UrlFetchApp.fetch(config.apiUrl + config.token + "/", data).getContentText());
}